@@ -28,7 +28,7 @@
 #include "gtksnapshot.h"
 #include "gtktypebuiltins.h"
 #include "gtkwidgetprivate.h"
-#include "gdkpixbufutilsprivate.h"
+#include "gdktextureutilsprivate.h"
 
 /**
  * GtkPicture:
@@ -173,8 +173,11 @@ gtk_picture_snapshot (GtkWidget   *widget,
             }
         }
 
-      x = (width - ceil (w)) / 2;
-      y = floor(height - ceil (h)) / 2;
+      w = ceil (w);
+      h = ceil (h);
+
+      x = (width - w) / 2;
+      y = floor(height - h) / 2;
 
       gtk_snapshot_save (snapshot);
       gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT (x, y));
@@ -212,7 +215,7 @@ gtk_picture_measure (GtkWidget      *widget,
     }
 
   style = gtk_css_node_get_style (gtk_widget_get_css_node (widget));
-  default_size = _gtk_css_number_value_get (style->icon->icon_size, 100);
+  default_size = gtk_css_number_value_get (style->icon->icon_size, 100);
 
   if (self->can_shrink)
     {
@@ -397,7 +400,7 @@ gtk_picture_class_init (GtkPictureClass *class)
   widget_class->measure = gtk_picture_measure;
 
   /**
-   * GtkPicture:paintable: (attributes org.gtk.Property.get=gtk_picture_get_paintable org.gtk.Property.set=gtk_picture_set_paintable)
+   * GtkPicture:paintable:
    *
    * The `GdkPaintable` to be displayed by this `GtkPicture`.
    */
@@ -407,7 +410,7 @@ gtk_picture_class_init (GtkPictureClass *class)
                            GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPicture:file: (attributes org.gtk.Property.get=gtk_picture_get_file org.gtk.Property.set=gtk_picture_set_file)
+   * GtkPicture:file:
    *
    * The `GFile` that is displayed or %NULL if none.
    */
@@ -417,7 +420,7 @@ gtk_picture_class_init (GtkPictureClass *class)
                            GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPicture:alternative-text: (attributes org.gtk.Property.get=gtk_picture_get_alternative_text org.gtk.Property.set=gtk_picture_set_alternative_text)
+   * GtkPicture:alternative-text:
    *
    * The alternative textual description for the picture.
    */
@@ -427,7 +430,7 @@ gtk_picture_class_init (GtkPictureClass *class)
                            GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPicture:keep-aspect-ratio: (attributes org.gtk.Property.get=gtk_picture_get_keep_aspect_ratio org.gtk.Property.set=gtk_picture_set_keep_aspect_ratio)
+   * GtkPicture:keep-aspect-ratio:
    *
    * Whether the GtkPicture will render its contents trying to preserve the aspect
    * ratio.
@@ -442,7 +445,7 @@ gtk_picture_class_init (GtkPictureClass *class)
                             G_PARAM_DEPRECATED);
 
   /**
-   * GtkPicture:can-shrink: (attributes org.gtk.Property.get=gtk_picture_get_can_shrink org.gtk.Property.set=gtk_picture_set_can_shrink)
+   * GtkPicture:can-shrink:
    *
    * If the `GtkPicture` can be made smaller than the natural size of its contents.
    */
@@ -452,7 +455,7 @@ gtk_picture_class_init (GtkPictureClass *class)
                             GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPicture:content-fit: (attributes org.gtk.Property.get=gtk_picture_get_content_fit org.gtk.Property.set=gtk_picture_set_content_fit)
+   * GtkPicture:content-fit:
    *
    * How the content should be resized to fit inside the `GtkPicture`.
    *
@@ -525,6 +528,9 @@ gtk_picture_new_for_paintable (GdkPaintable *paintable)
  * The pixbuf must not be modified after passing it to this function.
  *
  * Returns: a new `GtkPicture`
+ *
+ * Deprecated: 4.12: Use [ctor@Gtk.Picture.new_for_paintable] and
+ *   [ctor@Gdk.Texture.new_for_pixbuf] instead
  */
 GtkWidget*
 gtk_picture_new_for_pixbuf (GdkPixbuf *pixbuf)
@@ -645,7 +651,7 @@ gtk_picture_new_for_resource (const char *resource_path)
 }
 
 /**
- * gtk_picture_set_file: (attributes org.gtk.Method.set_property=file)
+ * gtk_picture_set_file:
  * @self: a `GtkPicture`
  * @file: (nullable): a `GFile`
  *
@@ -682,7 +688,7 @@ gtk_picture_set_file (GtkPicture *self,
 }
 
 /**
- * gtk_picture_get_file: (attributes org.gtk.Method.get_property=file)
+ * gtk_picture_get_file:
  * @self: a `GtkPicture`
  *
  * Gets the `GFile` currently displayed if @self is displaying a file.
@@ -779,6 +785,8 @@ gtk_picture_set_resource (GtkPicture *self,
  * See [ctor@Gtk.Picture.new_for_pixbuf] for details.
  *
  * This is a utility function that calls [method@Gtk.Picture.set_paintable].
+ *
+ * Deprecated: 4.12: Use [method@Gtk.Picture.set_paintable] instead
  */
 void
 gtk_picture_set_pixbuf (GtkPicture *self,
@@ -800,8 +808,22 @@ gtk_picture_set_pixbuf (GtkPicture *self,
     g_object_unref (texture);
 }
 
+static gboolean
+paintable_size_equal (GdkPaintable *one,
+                      GdkPaintable *two)
+{
+  if (one == NULL)
+    return two == NULL;
+  else if (two == NULL)
+    return FALSE;
+
+  return gdk_paintable_get_intrinsic_width (one) == gdk_paintable_get_intrinsic_width (two) &&
+         gdk_paintable_get_intrinsic_height (one) == gdk_paintable_get_intrinsic_height (two) &&
+         gdk_paintable_get_intrinsic_aspect_ratio (one) == gdk_paintable_get_intrinsic_aspect_ratio (two);
+}
+
 /**
- * gtk_picture_set_paintable: (attributes org.gtk.Method.set_property=paintable)
+ * gtk_picture_set_paintable:
  * @self: a `GtkPicture`
  * @paintable: (nullable): a `GdkPaintable`
  *
@@ -815,6 +837,8 @@ void
 gtk_picture_set_paintable (GtkPicture   *self,
                            GdkPaintable *paintable)
 {
+  gboolean size_changed;
+
   g_return_if_fail (GTK_IS_PICTURE (self));
   g_return_if_fail (paintable == NULL || GDK_IS_PAINTABLE (paintable));
 
@@ -826,6 +850,8 @@ gtk_picture_set_paintable (GtkPicture   *self,
   if (paintable)
     g_object_ref (paintable);
 
+  size_changed = !paintable_size_equal (self->paintable, paintable);
+ 
   gtk_picture_clear_paintable (self);
 
   self->paintable = paintable;
@@ -847,7 +873,8 @@ gtk_picture_set_paintable (GtkPicture   *self,
                           self);
     }
 
-  gtk_widget_queue_resize (GTK_WIDGET (self));
+  if (size_changed)
+    gtk_widget_queue_resize (GTK_WIDGET (self));
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_PAINTABLE]);
 
@@ -855,7 +882,7 @@ gtk_picture_set_paintable (GtkPicture   *self,
 }
 
 /**
- * gtk_picture_get_paintable: (attributes org.gtk.Method.get_property=paintable)
+ * gtk_picture_get_paintable:
  * @self: a `GtkPicture`
  *
  * Gets the `GdkPaintable` being displayed by the `GtkPicture`.
@@ -871,7 +898,7 @@ gtk_picture_get_paintable (GtkPicture *self)
 }
 
 /**
- * gtk_picture_set_keep_aspect_ratio: (attributes org.gtk.Method.set_property=keep-aspect-ratio)
+ * gtk_picture_set_keep_aspect_ratio:
  * @self: a `GtkPicture`
  * @keep_aspect_ratio: whether to keep aspect ratio
  *
@@ -900,7 +927,7 @@ gtk_picture_set_keep_aspect_ratio (GtkPicture *self,
 }
 
 /**
- * gtk_picture_get_keep_aspect_ratio: (attributes org.gtk.Method.get_property=keep-aspect-ratio)
+ * gtk_picture_get_keep_aspect_ratio:
  * @self: a `GtkPicture`
  *
  * Returns whether the `GtkPicture` preserves its contents aspect ratio.
@@ -920,7 +947,7 @@ gtk_picture_get_keep_aspect_ratio (GtkPicture *self)
 }
 
 /**
- * gtk_picture_set_can_shrink: (attributes org.gtk.Method.set_property=can-shrink)
+ * gtk_picture_set_can_shrink:
  * @self: a `GtkPicture`
  * @can_shrink: if @self can be made smaller than its contents
  *
@@ -952,7 +979,7 @@ gtk_picture_set_can_shrink (GtkPicture *self,
 }
 
 /**
- * gtk_picture_get_can_shrink: (attributes org.gtk.Method.get_property=can-shrink)
+ * gtk_picture_get_can_shrink:
  * @self: a `GtkPicture`
  *
  * Returns whether the `GtkPicture` respects its contents size.
@@ -968,7 +995,7 @@ gtk_picture_get_can_shrink (GtkPicture *self)
 }
 
 /**
- * gtk_picture_set_content_fit: (attributes org.gtk.Method.set_property=content-fit)
+ * gtk_picture_set_content_fit:
  * @self: a `GtkPicture`
  * @content_fit: the content fit mode
  *
@@ -1003,7 +1030,7 @@ gtk_picture_set_content_fit (GtkPicture    *self,
 }
 
 /**
- * gtk_picture_get_content_fit: (attributes org.gtk.Method.get_property=content-fit)
+ * gtk_picture_get_content_fit:
  * @self: a `GtkPicture`
  *
  * Returns the fit mode for the content of the `GtkPicture`.
@@ -1023,7 +1050,7 @@ gtk_picture_get_content_fit (GtkPicture *self)
 }
 
 /**
- * gtk_picture_set_alternative_text: (attributes org.gtk.Method.set_property=alternative-text)
+ * gtk_picture_set_alternative_text:
  * @self: a `GtkPicture`
  * @alternative_text: (nullable): a textual description of the contents
  *
@@ -1055,7 +1082,7 @@ gtk_picture_set_alternative_text (GtkPicture *self,
 }
 
 /**
- * gtk_picture_get_alternative_text: (attributes org.gtk.Method.get_property=alternative-text)
+ * gtk_picture_get_alternative_text:
  * @self: a `GtkPicture`
  *
  * Gets the alternative textual description of the picture.
